--[[

RandomCompanion
Picks a random vanity pet or mount and automatically chooses the best type of mount for the job.

Usage:
Bind a key from the keybinding menu or...
"/rc mount" to choose a random mount
"/rc mount ground" to choose a random ground mount in a flyable zone
"/rc pet" to choose a random vanity pet
"/rc autorecall" to toggle automatically recalling your vanity pet after resurrecting, changing zones, or taking flight paths
"/rc randomrecall" to toggle recalling a random vanity pet after resurrecting, changing zones, or taking flight paths
"/rc dismiss" to dismiss your current vanity pet
"/rc autodismiss" to toggle automatically dismissing your vanity pet when you are stealthed and flagged for PVP

TODO:
Button that shows your current pet/mount or a ? if you don't have one out
Change auto-dismiss/no-summon code to use spell IDs instead of spell names?
Make auto-recall not break meeting stone summons (might not be possible)
/pdismiss isn't always permanent?
Localization for Vashj'ir
Remove unneeded Northrend zones from localization

Changes:
2.1.1
Fixed using a ground mount in 4.0.3a Azeroth if you do not have a flight license.

2.1
Fixed a bug with the Celestial Steed on characters that are in a flyable area of Outland, but have not yet learned how to fly.
Changed how Cold Weather Flying and Flight Master's License are detected to a way that no longer needs localization. As a result, RandomCompanion should work better in more languages.
Added falling and death to the list of conditions that will temporarily prevent vanity pets from being auto-recalled.
Added a temporary hack for working around the Wintergrasp IsFlyableArea() bug.
Added a hack to check if bug mounts are usable in AQ40 due to a bug in WoW 4.0.1. Bug mounts will be skipped in AQ40 until the bug is fixed, and then they should automatically start working again.
Changed autorecall to wait 2 seconds before recalling your pet. This should prevent your GCD from getting locked by autorecall as often.
Fixed a bug with detecting snowballs in inventory.
Fixed a minor bug with DismissCompanion() and autorecall.
Tracked down a very annoying problem that was occasionally causing tons of errors while in combat...For reals this time.

2.0
Updated for version 4.0.1.
Added an options screen to the Interface Addons panel. Hello 2008!
Moved special mounts to use spellids instead of names for quicker and more effective localization.
All flying mounts are now in a single list since 4.0 allows all flying mounts to move at the same speed.
All ground mounts are now in a single list since 4.0 allows all ground mounts to move at the same speed.
Added Blazing Hippogryph support.
Added Abyssal Seahorse support.
Profession-based mount requirements should now be correctly detected if the profession is dropped and re-learned.
Non-ground mounts are now hardcoded since Blizzard saw fit to remove text that let RandomCompanion auto-detect new flying mounts. Unfortunately, this means that new non-ground mounts will require a new version of RandomCompanion.
Improved detection for current continent, which removes the need for zone name localization.
Changed the way Snowballs are found in inventory.
Added invisibility to the list of buffs which will stop AutoRecall for reals this time.
Added Graccu's Mince Meat Fruitcake to the list of buffs which will stop AutoRecall.
Tracked down a very annoying problem that was occasionally causing tons of errors while in combat.

1.7.4
Changed how the COMPANION_UPDATE event is handled to not happen so often.
Added flight form and swift flight form to the list of buffs which will stop AutoRecall.
Added support for the Big Love Rocket and Invincible. (untested, they should work as extreme flyers, so please let me know if these work properly)
Added support for the Celestial Steed and the upcoming X-53 Touring Rocket. (untested)
Fixed a small bug with scaling mounts.
Made a change to how the weight slider works so it should no longer cause problems when you move the slider and then quickly choose another mount/pet.

1.7.3
Lowered the amount of time between moving the weight slider and saving the weight.
Updated for version 3.3.

1.7.2
When you are in a vehicle, you will exit the vehicle when you trying to mount.
Added macro conditional parsing to /rc. Example: "/rc [button:1] mount; [button:2] pet" will summon your mount when you left-click the macro, or your pet when you right-click.

1.7.1
Fixed a problem with initialization that was introduced in 1.7.
Fixed a problem where randomly summoning the same vanity pet that you already have out will instead dismiss your current pet.
Added invisibility to the list of buffs which will stop AutoRecall.

1.7
Consolidated all /commands under /rc or /randomcompanion. Old /commands will remain in place for now for backward compatibility. Type "/rc" for a full list of available commands.
Added "/rc autodismiss" to toggle automatically dismissing your vanity pet when you are stealthed and flagged for PVP.
Added "/rc randomrecall" to toggle recalling a random vanity pet after resurrecting, changing zones, or taking flight paths.
Fixed some oddities with AutoRecall.
AutoRecall (and RandomRecall) now recalls your vanity pet when you log in.
Added support for the Onyxian Drake.

1.6.2
Added localization for Krasus' Landing and Underbelly for the German client. Thanks, Herekren.
Added localization for Krasus' Landing for the French and Russian clients. Still need Underbelly translations for these clients.
Added support for the Rusted and Ironbound Proto-Drakes.
Updated name of Headless Horseman's Mount because it changed slightly in 3.2.
Removed Dalaran as special flyable zone because IsFlyableArea() finally works as expected there.
Wintergrasp is now flyable when a battle is not in progress.

1.6.1
Fixed a bug with the weight UI that happened after changing zones

1.6
UI to disable mounts/pets that you don't like or choose your favorite mounts/pets more often has finally been added! Just open your Mount/Pet window and adjust the slider.
Black Qiraji Battle Tank should now work properly both inside and outside of Ahn'Qiraj.

1.5.1
Fixed a bug that detects other mounts as swimmers if you have a sea turtle.

1.5
RandomCompanion will now choose a swimming mount (the new sea turtle) or a ground mount if you are swimming.
Added "/rpet dismiss" and "/pdismiss" to dismiss your current vanity pet.
Vanity pets are automatically re-summoned after losing them for any reason. "/rpet autorecall" to toggle this feature.

1.4
Added Russian localization for Dragonblight.
Fixed a bug with flyable detection in the sewer pipe section of Dalaran.
Fixed a bug with flyable detection in part of Krasus' Landing in Dalaran.
Added flyable detection to some areas on the outskirts of Dalaran. It's not perfect, but it's better than it used to be.
Added /rcreload in case you ever want to re-initialize for some reason.
Added macro conditional parsing such as [modifier:alt] or [combat] to /rmount.
Added checks to make sure you have engineering for the engineering mounts and tailoring for the tailoring mounts. Does not check skill level, so if you re-learn your old tradeskill, your not-yet-usable mounts may be chosen.
Vanity pets are automatically re-summoned after resurrecting, changing zones, or taking flight paths. "/rpet autorecall" to toggle this feature.
LOTS of new debug messages (/rcdebug to enable), so if you're having any sort of problems, turn on debug mode and report them to http://wow.curse.com/downloads/wow-addons/details/randomcompanion.aspx
Added a check for snowballs before summoning winter pets (Winter Reindeer, etc.)

1.3
Fixed bug mount prioritization in AQ40 - May not work properly in non-English clients yet.
Changed the German localization to support an oddity with the Turbogetriebene Flugmaschine.
Added /rcstatus to get RandomCompanion status and /rcdebug to toggle debug mode.
Fixed a bug with scaling flying mounts not getting added to the slow flyers list.
Uncached mounts and pets should be handled a lot better.
Fixed a bug with flyable detection inside Dalaran.

]]--

local VERSION = GetAddOnMetadata("RandomCompanion", "Version");

RandomCompanion = {
	mounts = {
		unweighted = {
			ground = {};
			flying = {};
			swimmers = {};
			vashjir = {};
			bugs = {};
		};
		weighted = {
			ground = {};
			flying = {};
			swimmers = {};
			vashjir = {};
			bugs = {};
		};
	};
	critters = {
		weighted = {};
		unweighted = {};
		winter = {};
	};
	Engineering = 0;
	Tailoring = 0;
	zones = {
		kalimdor = {};
		eastern_kingdoms = {};
		outland = {};
		northrend = {};
	};
};

local function debugmsg(msg, debuglevel)
	if debuglevel == nil then
		debuglevel = 1;
	end
	if tonumber(debuglevel) then
		local debuglevel = tonumber(debuglevel);
	else	
		local debuglevel = 1;
	end
	if RandomCompanion_settings.DEBUG and (debuglevel >= tonumber(RandomCompanion_settings.DEBUGLevel)) then
		DEFAULT_CHAT_FRAME:AddMessage('RandomCompanion Debug[' .. debuglevel .. ']: ' .. msg);
	end
end

function RandomCompanion.Event(self, event, ...)
	if event == "PLAYER_LOGIN" then
		RandomCompanion.Initialize(true);
	elseif event == "COMPANION_LEARNED" then
		RandomCompanion.Initialize(false);
	elseif event == "COMPANION_UPDATE" then
		if ... == "CRITTER" then
			RandomCompanion.SetActiveCompanions();
		end
	elseif event == "PLAYER_ENTERING_WORLD" or event == "PLAYER_CONTROL_GAINED" or event == "PLAYER_UNGHOST" or event == "PLAYER_ALIVE" then
		RandomCompanion.RecallCritter();
	else
		debugmsg("Unknown event triggered.");
	end
end

function RandomCompanion.Help()
	--For backwards compatibility with old slash commands
	RandomCompanion.Slash("help");
end

function RandomCompanion.Initialize(showstatus)
	RC_REQUIRE_RELOAD = false;
	
	if not RandomCompanion_settings then
		RandomCompanion_settings = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()] then
		RandomCompanion_settings[GetRealmName()] = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()][UnitName("player")] then
		RandomCompanion_settings[GetRealmName()][UnitName("player")] = {};
	end
	
	if not RandomCompanion_settings.DEBUG then
		RandomCompanion_settings.DEBUG = false;
	end
	
	if RandomCompanion_settings.DEBUGLevel == nil then
		RandomCompanion_settings.DEBUGLevel = 1;
	end
	
	if RandomCompanion_settings.AutoRecall == nil then
		RandomCompanion_settings.AutoRecall = true;
		if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
			RandomCompanion.OnUpdateTime = GetTime() + 2;
			RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
		else
			RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
		end
	end
	
	if RandomCompanion_settings.AutoDismiss == nil then
		RandomCompanion_settings.AutoDismiss = true;
		if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
			RandomCompanion.OnUpdateTime = GetTime() + 2;
			RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
		else
			RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
		end
	end
	
	if RandomCompanion_settings.RandomRecall == nil then
		RandomCompanion_settings.RandomRecall = false;
	end
	
	if not RandomCompanion_settings[GetRealmName()] then
		RandomCompanion_settings[GetRealmName()] = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()][UnitName("player")] then
		RandomCompanion_settings[GetRealmName()][UnitName("player")] = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()][UnitName("player")].weights then
		RandomCompanion_settings[GetRealmName()][UnitName("player")].weights = {};
	end
	
	RandomCompanion.mounts = {
		unweighted = {
			ground = {};
			flying = {};
			swimmers = {};
			vashjir = {};
			bugs = {};
		};
		weighted = {
			ground = {};
			flying = {};
			swimmers = {};
			vashjir = {};
			bugs = {};
		};
	};
	RandomCompanion.critters = {
		weighted = {};
		unweighted = {};
		winter = {};
	};
	
	RandomCompanion.zones = {
		kalimdor = {};
		eastern_kingdoms = {};
		outland = {};
		northrend = {};
	};
	
	local function LoadZones(c, ...)
		for i=1,select('#', ...),1 do
			c[i] = select(i,...)
		end
	end
	
	LoadZones(RandomCompanion.zones.kalimdor, GetMapZones(1));
	LoadZones(RandomCompanion.zones.eastern_kingdoms, GetMapZones(2));
	LoadZones(RandomCompanion.zones.outland, GetMapZones(3));
	LoadZones(RandomCompanion.zones.northrend, GetMapZones(4));
	
	if not RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
		RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter = nil;
		RandomCompanion.SetActiveCompanions();
	end
	
	RandomCompanion.Engineering = 0;
	RandomCompanion.Tailoring = 0;
	
	local prof1, prof2, arch, fish, cook, firstAid = GetProfessions();
	if (prof1) then
		local name, texture, rank, maxRank, numSpells, spelloffset, skillLine = GetProfessionInfo(prof1);
		debugmsg("Profession 1: " .. name .. " rank: " .. rank, 1);
		if (name == RC_STRING_ENGINEERING) then
			RandomCompanion.Engineering = rank;
		elseif (name == RC_STRING_TAILORING) then
			RandomCompanion.Tailoring = rank;
		end
	end
	
	if (prof2) then
		local name, texture, rank, maxRank, numSpells, spelloffset, skillLine = GetProfessionInfo(prof2);
		debugmsg("Profession 2: " .. name .. " rank: " .. rank, 1);
		if (name == RC_STRING_ENGINEERING) then
			RandomCompanion.Engineering = rank;
		elseif (name == RC_STRING_TAILORING) then
			RandomCompanion.Tailoring = rank;
		end
	end
	
	CreateFrame("GameTooltip", "RandomCompanionScanningTooltip", UIParent, "GameTooltipTemplate");
	RandomCompanionScanningTooltip:AddFontStrings(
		RandomCompanionScanningTooltip:CreateFontString( "$parentTextLeft1", nil, "GameTooltipText" ),
		RandomCompanionScanningTooltip:CreateFontString( "$parentTextRight1", nil, "GameTooltipText" )
	);

	for index = 1, GetNumCompanions("MOUNT") do
		local aq = false;
		local swimmer = false;
		local outland = false;
		local northrend = false;
		local very = false;
		local extremely = false;
		local fast = false;
		local engineering = 0;
		local tailoring = 0;
		local creatureid, name, spellid, icon, active = GetCompanionInfo("MOUNT", index);
		
		if name then
			GameTooltip_SetDefaultAnchor(RandomCompanionScanningTooltip, UIParent) 
			RandomCompanionScanningTooltip:SetHyperlink("spell:"..spellid)
			
			local text = '';
			for i = 1, RandomCompanionScanningTooltip:NumLines() do
				text = text .. _G["RandomCompanionScanningTooltipTextLeft"..i]:GetText();
			end
			
			if string.find(text, RC_STRING_AHN_QIRAJ) then
--				aq = true;
			end
			if string.find(text, RC_STRING_SWIMMER) then
--				swimmer = true;
			end
			if string.find(text, RC_STRING_OUTLAND) then
--				outland = true;
			end
			if string.find(text, RC_STRING_NORTHREND) then
--				northrend = true;
			end
			if string.find(text, RC_STRING_VERY) then
--				very = true;
			end
			if string.find(text, RC_STRING_EXTREMELY) then
--				extremely = true;
			end
			if string.find(text, RC_STRING_FAST) then
--				fast = true;
			end
			if string.find(string.lower(text), string.lower(RC_STRING_ENGINEERING)) then
				if (string.find(text, RC_STRING_ENGINEERING_REQUIREMENT .. " ")) then
					local professiontext = string.sub(text, string.find(text, RC_STRING_ENGINEERING_REQUIREMENT .. " ") + string.len(RC_STRING_ENGINEERING_REQUIREMENT) + 1);
					professionrequirement = string.sub(professiontext, 0, string.find(professiontext, " "));
					debugmsg(name .. " requires engineering level " .. professionrequirement, 1);
					engineering = tonumber(professionrequirement);
				else
					debugmsg("Engineering required, but could not determine required skill level.", 1);
					engineering = 1;
				end
			end
			if string.find(string.lower(text), string.lower(RC_STRING_TAILORING)) then
				if (string.find(text, RC_STRING_TAILORING_REQUIREMENT .. " ")) then
					local professiontext = string.sub(text, string.find(text, RC_STRING_TAILORING_REQUIREMENT .. " ") + string.len(RC_STRING_TAILORING_REQUIREMENT) + 1);
					professionrequirement = string.sub(professiontext, 0, string.find(professiontext, " "));
					debugmsg(name .. " requires tailoring level " .. professionrequirement, 1);
					tailoring = tonumber(professionrequirement);
				else
					debugmsg("Tailoring required, but could not determine required skill level.", 1);
					tailoring = 1;
				end
			end
			
			if not RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[name] then
				RandomCompanion.SetWeight(name, 1);
			end
			
			local weight = RandomCompanion.GetWeight(name);
			
			if (not engineering or RandomCompanion.Engineering >= engineering) and (not tailoring or RandomCompanion.Tailoring >= tailoring) then
				if swimmer or tContains(RC_MOUNT_IDS.swimming, spellid) then
					local nextindex = #RandomCompanion.mounts.unweighted.swimmers + 1;
					RandomCompanion.mounts.unweighted.swimmers[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				elseif tContains(RC_MOUNT_IDS.vashjir, spellid) then
					local nextindex = #RandomCompanion.mounts.unweighted.vashjir + 1;
					RandomCompanion.mounts.unweighted.vashjir[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				elseif spellid == RC_SPELLID_BLACK_QIRAJI_BATTLE_TANK or spellid == RC_SPELLID_ULTRAMARINE_QIRAJI_BATTLE_TANK then
					local nextindex = #RandomCompanion.mounts.unweighted.ground + 1;
					RandomCompanion.mounts.unweighted.ground[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
					local nextindex = #RandomCompanion.mounts.unweighted.bugs + 1;
					RandomCompanion.mounts.unweighted.bugs[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				elseif aq or tContains(RC_MOUNT_IDS.bugs, spellid) then
					local nextindex = #RandomCompanion.mounts.unweighted.bugs + 1;
					RandomCompanion.mounts.unweighted.bugs[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				elseif tContains(RC_MOUNT_IDS.scaling, spellid) then
					local nextindex = #RandomCompanion.mounts.unweighted.flying + 1;
					RandomCompanion.mounts.unweighted.flying[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
					local nextindex = #RandomCompanion.mounts.unweighted.ground + 1;
					RandomCompanion.mounts.unweighted.ground[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				elseif outland or northrend or tContains(RC_MOUNT_IDS.flying, spellid) then
					local nextindex = #RandomCompanion.mounts.unweighted.flying + 1;
					RandomCompanion.mounts.unweighted.flying[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
					--[[
					--Uncomment this if flying mounts are ever usable on the ground again like on the 4.0 PTR
					local nextindex = #RandomCompanion.mounts.unweighted.ground + 1;
					RandomCompanion.mounts.unweighted.ground[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
					]]--
				elseif not outland and not northrend then
					local nextindex = #RandomCompanion.mounts.unweighted.ground + 1;
					RandomCompanion.mounts.unweighted.ground[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				else
					-- This should never happen, but just in case...
					local options = "";
					if very then
						options = options .. " very";
					end
					if extremely then
						options = options .. " extremely";
					end
					if fast then
						options = options .. " fast";
					end
					if outland then
						options = options .. " outland";
					end
					if northrend then
						options = options .. " northrend";
					end
					if options == "" then
						options = " none";
					end
					DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion " .. VERSION .. ":" .. name .. " mount type could not be detected. (spellid:" .. spellid .. " options:" .. options .. ")");
					DEFAULT_CHAT_FRAME:AddMessage("Please leave a comment about this at http://wow.curse.com/downloads/wow-addons/details/randomcompanion.aspx");
				end
			else
				debugmsg(name .. " requires a tradeskill that you no longer have.", 8);
			end
			
			if weight ~= 1 then
				debugmsg("Mount " .. name .. " has non-default weight: " .. weight, 2);
			end
			
			if weight > 0 then
				for counter = 1, weight do
					if swimmer or tContains(RC_MOUNT_IDS.swimming, spellid) then
						local nextindex = #RandomCompanion.mounts.weighted.swimmers + 1;
						RandomCompanion.mounts.weighted.swimmers[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					elseif tContains(RC_MOUNT_IDS.vashjir, spellid) then
						local nextindex = #RandomCompanion.mounts.weighted.vashjir + 1;
						RandomCompanion.mounts.weighted.vashjir[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					elseif spellid == RC_SPELLID_BLACK_QIRAJI_BATTLE_TANK then
						local nextindex = #RandomCompanion.mounts.weighted.ground + 1;
						RandomCompanion.mounts.weighted.ground[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
						local nextindex = #RandomCompanion.mounts.weighted.bugs + 1;
						RandomCompanion.mounts.weighted.bugs[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					elseif aq or tContains(RC_MOUNT_IDS.bugs, spellid) then
						local nextindex = #RandomCompanion.mounts.weighted.bugs + 1;
						RandomCompanion.mounts.weighted.bugs[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					elseif tContains(RC_MOUNT_IDS.scaling, spellid) then
						local nextindex = #RandomCompanion.mounts.weighted.flying + 1;
						RandomCompanion.mounts.weighted.flying[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
						local nextindex = #RandomCompanion.mounts.weighted.ground + 1;
						RandomCompanion.mounts.weighted.ground[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					elseif outland or northrend or tContains(RC_MOUNT_IDS.flying, spellid) then
						local nextindex = #RandomCompanion.mounts.weighted.flying + 1;
						RandomCompanion.mounts.weighted.flying[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
						--[[
						--Uncomment this if flying mounts are ever usable on the ground again like on the 4.0 PTR
						local nextindex = #RandomCompanion.mounts.weighted.ground + 1;
						RandomCompanion.mounts.weighted.ground[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
						]]--
					elseif not outland and not northrend then
						local nextindex = #RandomCompanion.mounts.weighted.ground + 1;
						RandomCompanion.mounts.weighted.ground[nextindex] = {
							creatureid = creatureid;
							name = name;
							index = index;
							spellid = spellid;
							icon = icon;
							active = active;
						}
					end
				end
			end
		else
			--This seems to happen when the companion is not in the cache
			RC_REQUIRE_RELOAD = true;
			debugmsg(index .. " index mount has no name?");
		end
	end
	
	RandomCompanionScanningTooltip:Hide()
	
	for index = 1, GetNumCompanions("CRITTER") do
		local creatureid, name, spellid, icon, active = GetCompanionInfo("CRITTER", index);
		if name then
			if not RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[name] then
				RandomCompanion.SetWeight(name, 1);
			end
			local weight = RandomCompanion.GetWeight(name);
			
			local nextindex = #RandomCompanion.critters.unweighted + 1;
			RandomCompanion.critters.unweighted[nextindex] = {
				creatureid = creatureid;
				name = name;
				index = index;
				spellid = spellid;
				icon = icon;
				active = active;
			}
			
			if weight ~= 1 then
				debugmsg("Critter " .. name .. " has non-default weight: " .. weight, 2);
			end
			
			if weight > 0 then
				for counter = 1, weight do
					local nextindex = #RandomCompanion.critters.weighted + 1;
					RandomCompanion.critters.weighted[nextindex] = {
						creatureid = creatureid;
						name = name;
						index = index;
						spellid = spellid;
						icon = icon;
						active = active;
					}
				end
			end
			
			--Check to see if this critter requires a snowball to summon
			GameTooltip_SetDefaultAnchor(RandomCompanionScanningTooltip, UIParent) 
			RandomCompanionScanningTooltip:SetHyperlink("spell:"..spellid)
			
			for i = 1, RandomCompanionScanningTooltip:NumLines() do
				local text = _G["RandomCompanionScanningTooltipTextLeft"..i]:GetText();
				
				if string.find(string.lower(text), string.lower(RC_STRING_SNOWBALL)) then
					RandomCompanion.critters.winter[index] = true;
					debugmsg(name .. " (index: " .. index .. ") is a winter pet that requires a snowball to summon.", 3);
					break;
				end
			end
			
			RandomCompanionScanningTooltip:Hide()
		else
			--This seems to happen when the companion is not in the cache
			RC_REQUIRE_RELOAD = true;
			debugmsg(index .. " index critter has no name?");
		end
	end
	
	RandomCompanion.SetActiveCompanions();
	
	if showstatus then
		RandomCompanion.Status();
	end
end

function RandomCompanion.Debug()
	--For backwards compatibility with old slash commands
	RandomCompanion.Slash("debug");
end

function RandomCompanion.Status()
	local loadmessage = "";
	
	if #RandomCompanion.mounts.unweighted.flying > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. #RandomCompanion.mounts.unweighted.flying .. " flyers";
	end
	if #RandomCompanion.mounts.unweighted.ground > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. #RandomCompanion.mounts.unweighted.ground .. " grounders";
	end
	if #RandomCompanion.mounts.unweighted.bugs > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. #RandomCompanion.mounts.unweighted.bugs .. " bugs";
	end
	if #RandomCompanion.mounts.unweighted.swimmers > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. #RandomCompanion.mounts.unweighted.swimmers .. " swimmers";
	end
	if #RandomCompanion.mounts.unweighted.vashjir > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. #RandomCompanion.mounts.unweighted.vashjir .. " Vashj'ir mounts";
	end
	if #RandomCompanion.critters.unweighted > 0 then
		if loadmessage ~= "" then
			loadmessage = loadmessage .. ", ";
		end
		loadmessage = loadmessage .. #RandomCompanion.critters.unweighted .. " critters";
	end
	
	if loadmessage ~= "" then
		loadmessage = " with " .. loadmessage .. ".";
	end
	
	if RandomCompanion_settings.RandomRecall then
		loadmessage = loadmessage .. " Vanity pets will be automatically randomly re-summoned when possible.";
	elseif RandomCompanion_settings.AutoRecall then
		loadmessage = loadmessage .. " Vanity pets will be automatically re-summoned when possible.";
	end
	
	if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
		RandomCompanion.OnUpdateTime = GetTime() + 2;
		RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
	else
		RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
	end
	
	if RandomCompanion_settings.AutoDismiss then
		loadmessage = loadmessage .. " Vanity pets will be automatically dismissed when entering stealth while flagged for PVP.";
		if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
			RandomCompanion.OnUpdateTime = GetTime() + 2;
			RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
		else
			RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
		end
	end
	
	if RC_REQUIRE_RELOAD then
		loadmessage = loadmessage .. " RandomCompanion will reload when next companion is summoned.";
	end

	DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion " .. VERSION .. " loaded" .. loadmessage);
end

function RandomCompanion.Mount(cmd)
	local flyable;
	
	if (cmd) then
		--Parse any macro conditionals such as [modifier:ctrl] or [combat]
		cmd = SecureCmdOptionParse(cmd);
	end
	
	if RC_REQUIRE_RELOAD then
		--Re-initialize if one of the pet or mount names was not available in the initial load
		debugmsg("Re-initializing RandomCompanion", 1);
		RandomCompanion.Initialize();
	end
	
	if IsMounted() then
		Dismount();
	elseif CanExitVehicle() then
		VehicleExit();
	else
		local inInstance, instanceType = IsInInstance();
		if IsSwimming() and (GetRealZoneText() == RC_STRING_VASHJ_IR or GetRealZoneText() == RC_STRING_ABYSSAL_DEPTHS or GetRealZoneText() == RC_STRING_KELP_THAR_FOREST or GetRealZoneText() == RC_STRING_SHIMMERING_EXPANSE) then
			debugmsg("Currently swimming in Vash'jir - Mount preference: vashjir, swimmers, ground", 3);
			if #RandomCompanion.mounts.weighted.vashjir > 0 then
				mountlist = "vashjir";
			elseif #RandomCompanion.mounts.weighted.swimmers > 0 then
				mountlist = "swimmers";
			elseif #RandomCompanion.mounts.weighted.ground > 0 then
				mountlist = "ground";
			else
				mountlist = nil;
			end
		elseif IsSwimming() then
			debugmsg("Currently swimming - Mount preference: swimming, ground", 3);
			if #RandomCompanion.mounts.weighted.swimmers > 0 then
				mountlist = "swimmers";
			elseif #RandomCompanion.mounts.weighted.ground > 0 then
				mountlist = "ground";
			else
				mountlist = nil;
			end
		elseif GetRealZoneText() == RC_STRING_TEMPLE_OF_AHN_QIRAJ and inInstance then
			debugmsg("Currently in Temple of Ahn'Qiraj - Mount preference: bugs, ground", 3);
			if #RandomCompanion.mounts.weighted.bugs > 0 then
				if IsUsableSpell(RandomCompanion.mounts.weighted.bugs[1].spellid) then --Workaround for 4.0.1 bug...See if bug mounts are actually usable
					mountlist = "bugs";
				elseif #RandomCompanion.mounts.weighted.ground > 0 then
					debugmsg("Currently in Temple of Ahn'Qiraj, but bug mounts are not usable.", 2);
					mountlist = "ground";
				else
					mountlist = nil;
				end
			elseif #RandomCompanion.mounts.weighted.ground > 0 then
				mountlist = "ground";
			else
				mountlist = nil;
			end
		elseif RandomCompanion.IsFlyableArea() and cmd ~= "ground" then
			debugmsg("Currently in a flyable area - Mount preference: flying, ground", 3);
			if #RandomCompanion.mounts.weighted.flying > 0 then
				mountlist = "flying";
			elseif #RandomCompanion.mounts.weighted.ground > 0 then
				mountlist = "ground";
			else
				mountlist = nil;
			end
		else
			debugmsg("Mount preference: ground only", 3);
			if #RandomCompanion.mounts.weighted.ground > 0 then
				mountlist = "ground";
			else
				mountlist = nil;
			end
		end
		
		if mountlist then
			local mountindex = random(#RandomCompanion.mounts.weighted[mountlist]);
			debugmsg("Mounting " .. RandomCompanion.mounts.weighted[mountlist][mountindex].name .. " from " .. mountlist .." list. Index is " .. RandomCompanion.mounts.weighted[mountlist][mountindex].index, 10);
			CallCompanion("MOUNT", RandomCompanion.mounts.weighted[mountlist][mountindex].index);
		else
			if GetNumCompanions("MOUNT") > 0 then
				--We have mounts, but they aren't in the mount list. The most likely issue is that no mounts were in the cache when RandomCompanion.Initialize() ran, so let's run it again.
				if not RC_REINITIALIZED then
					debugmsg("Re-initializing because mount names weren't cached yet.");
					RC_REINITIALIZED = true; --We don't want an infinite loop, do we?
					RandomCompanion.Initialize();
					RandomCompanion.Mount(cmd);
				else
					RC_REINITIALIZED = nil;
				end
			else
				UIErrorsFrame:AddMessage("You have no mounts", 1.0, 0.0, 0.0, 53, 5);
			end
		end
	end
end

function RandomCompanion.IsFlyableArea()
	local expert_riding = IsUsableSpell(GetSpellInfo(RC_SPELLID_EXPERT_RIDING)) or 0;
	local artisan_riding = IsUsableSpell(GetSpellInfo(RC_SPELLID_ARTISAN_RIDING)) or 0;
	local master_riding = IsUsableSpell(GetSpellInfo(RC_SPELLID_MASTER_RIDING)) or 0;
	local cold_flying = IsUsableSpell(GetSpellInfo(RC_SPELLID_COLD_FLYING)) or 0;
	local flight_license = IsUsableSpell(GetSpellInfo(RC_SPELLID_FLIGHT_LICENSE)) or 0;
	
	debugmsg("Expert Riding: " .. tonumber(expert_riding) .. " Artisan Riding: " .. tonumber(artisan_riding) .. " Master Riding: " .. tonumber(master_riding) .. " Cold Flying: " .. tonumber(cold_flying) .. " Flight License: " .. tonumber(flight_license), 2);
	
	if expert_riding ~= 0 or artisan_riding ~= 0 or master_riding ~= 0 then
		if IsFlyableArea() then
			if tContains(RandomCompanion.zones.northrend, GetRealZoneText()) then
				debugmsg("Currently in Northrend", 2);
				
				if cold_flying ~= 0 then
					return true;
				else
					return false;
				end
			elseif tContains(RandomCompanion.zones.kalimdor, GetRealZoneText()) or tContains(RandomCompanion.zones.eastern_kingdoms, GetRealZoneText()) then
				debugmsg("Currently in Azeroth", 2);
				
				if flight_license ~= 0 then
					return true;
				else
					return false;
				end
			else
				return true;
			end
		elseif GetRealZoneText() == RC_STRING_WINTERGRASP then
			debugmsg("Currently in Wintergrasp", 2);
			
			if cold_flying ~= 0 and GetWintergraspWaitTime() ~= nil then
				return true;
			else
				return false;
			end
		else
			return false;
		end
	else
		return false;
	end
end

function RandomCompanion.Pet(cmd)
	if cmd and string.lower(cmd) == "autorecall" then
		--For backwards compatibility with old slash commands
		RandomCompanion.Slash(cmd);
	else
		if RC_REQUIRE_RELOAD then
			--Re-initialize if one of the pet or mount names was not available in the initial load
			debugmsg("Re-initializing RandomCompanion");
			RandomCompanion.Initialize();
		end
		
		if #RandomCompanion.critters.weighted > 0 then
			local petindex = nil;
			local havesnowball = GetItemCount(17202); --Snowball item id is 17202
			
			if havesnowball > 0 then
				debugmsg("Found a snowball in inventory. Winter pets may be summoned.", 3);
				petindex = random(#RandomCompanion.critters.weighted);
			else
				debugmsg("No snowballs in inventory. Winter pets may not be summoned.", 3);
				--Loop through 5 times trying to find a critter that doesn't require a snowball
				for counter = 1, 5 do 
					petindex = random(#RandomCompanion.critters.weighted);
					local companionindex = RandomCompanion.critters.weighted[petindex].index;
					if not RandomCompanion.critters.winter[companionindex] then
						debugmsg("Found a pet which does not require a snowball at index " .. RandomCompanion.critters.weighted[petindex].index, 3);
						break;
					else
						debugmsg("Skipping index " .. RandomCompanion.critters.weighted[petindex].index .. " (" .. RandomCompanion.critters.weighted[petindex].name .. ") because it requires a snowball.", 3);
					end
				end
			end
			
			--Only dismiss if there is an active critter and it's the same as the one that's about to be summoned, otherwise there are problems with RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter getting removed when you can't actually re-summon your pet, ie when dead
			if RandomCompanion.GetActiveCompanion("CRITTER") and RandomCompanion.GetActiveCompanion("CRITTER") == RandomCompanion.critters.weighted[petindex].index then
				DismissCompanion("CRITTER");
				debugmsg("Dismissing because critter is already active", 2);
			end
			
			debugmsg("Summoning " .. RandomCompanion.critters.weighted[petindex].name .. ". Index is " .. RandomCompanion.critters.weighted[petindex].index, 10);
			CallCompanion("CRITTER", RandomCompanion.critters.weighted[petindex].index);
		elseif GetNumCompanions("CRITTER") > 0 then
			--We have pets, but they aren't in the pet list. The most likely issue is that no pets were in the cache when RandomCompanion.Initialize() ran, so let's run it again.
			if not RC_REINITIALIZED then
				debugmsg("Re-initializing because pet names weren't cached yet.");
				RC_REINITIALIZED = true; --We don't want an infinite loop, do we?
				RandomCompanion.Initialize();
				RandomCompanion.Pet();
			else
				RC_REINITIALIZED = nil;
			end
		else
			UIErrorsFrame:AddMessage("You have no vanity pets", 1.0, 0.0, 0.0, 53, 5);
		end
	end
end

function RandomCompanion.PetDismiss()
	DismissCompanion("CRITTER");
end

function RandomCompanion.SetWeight(companion, value)
	if not RandomCompanion_settings then
		RandomCompanion_settings = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()] then
		RandomCompanion_settings[GetRealmName()] = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()][UnitName("player")] then
		RandomCompanion_settings[GetRealmName()][UnitName("player")] = {};
	end
	
	if not RandomCompanion_settings[GetRealmName()][UnitName("player")].weights then
		RandomCompanion_settings[GetRealmName()][UnitName("player")].weights = {};
	end
	
	RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[companion] = value;
	debugmsg("RandomCompanion_settings["..GetRealmName().."]["..UnitName("player").."].weights["..companion.."] = "..RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[companion], 1);
end

function RandomCompanion.GetWeight(companion)
	if RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[companion] then
		return RandomCompanion_settings[GetRealmName()][UnitName("player")].weights[companion];
	else
		return 1;
	end
end

function RandomCompanion.GetActiveCompanion(type)
	local activecompanion = nil;
	for index = 1, GetNumCompanions(type) do
		local creatureid, name, spellid, icon, active = GetCompanionInfo(type, index);
		if active then
			debugmsg("Active " .. string.lower(type) .. " is index " .. index, 1);
			activecompanion = index;
		end
	end
	return activecompanion;
end

function RandomCompanion.SetActiveCompanions()
	local activecritter = RandomCompanion.GetActiveCompanion("CRITTER");
	if activecritter and activecritter ~= RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
		RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter = activecritter;
		debugmsg("Setting active critter to index " .. activecritter, 2);
	end
	if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
		RandomCompanion.OnUpdateTime = GetTime() + 2;
		RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
	else
		RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
	end
end

function RandomCompanion.RecallCritter()
	if RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter and RandomCompanion_settings.AutoRecall and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
		local creatureid, name, spellid, icon, active = GetCompanionInfo("CRITTER", RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter);
		if not active then
			if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
				RandomCompanion.OnUpdateTime = GetTime() + 2;
				RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
			else
				RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
			end
			debugmsg("Automatically recalling critter from index " .. RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter .. " in 2 seconds", 4);
		end
	end
end

function RandomCompanion.OnUpdate()
	if RandomCompanion.OnUpdateTime == nil or GetTime() > RandomCompanion.OnUpdateTime then
		if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
			RandomCompanion.OnUpdateTime = GetTime() + 2;
			RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
		else
			RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
		end
		
		if (RandomCompanion.RecallTime == nil or GetTime() > RandomCompanion.RecallTime) and (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
			debugmsg("Checking to recall pets", 1);
			local creatureid, name, spellid, icon, active = GetCompanionInfo("CRITTER", RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter);
			
			if not active then
				--Check to see if you're doing something that will break when recalling a critter
				local eating = UnitBuff("player", RC_STRING_EATING);
				local drinking = UnitBuff("player", RC_STRING_DRINKING);
				local graccu = UnitBuff("player", RC_STRING_GRACCU);
				local stealth = UnitBuff("player", RC_STRING_STEALTH);
				local prowl = UnitBuff("player", RC_STRING_PROWL);
				local shadowmeld = UnitBuff("player", RC_STRING_SHADOWMELD);
				local invisibility = UnitBuff("player", RC_STRING_INVISIBILITY);
				local flightform = UnitBuff("player", RC_STRING_FLIGHT_FORM);
				local swiftflightform = UnitBuff("player", RC_STRING_SWIFT_FLIGHT_FORM);
				local combat = UnitAffectingCombat("player");
				local vehicle = UnitInVehicle("player");
				local casting, _, _, _, _, _ = UnitCastingInfo("player");
				local channeling, _, _, _, _, _ = UnitChannelInfo("player");
				local looting = GetNumLootItems();
				local dead = UnitIsDeadOrGhost("player");
				local falling = IsFalling();
				local mounted = IsMounted();
				
				if not dead and not falling and looting == 0 and not eating and not drinking and not graccu and not combat and not casting and not channeling and not mounted and not vehicle and not stealth and not prowl and not shadowmeld and not invisibility and not flightform and not swiftflightform then
					if RandomCompanion.NotBusy then 
						--You haven't been busy for 2 seconds, so it should be safe to recall your critter
						RandomCompanion.NotBusy = nil;
						RandomCompanion.RecallTime = GetTime() + 2;
						debugmsg("Recalling critter now", 4);
						if RandomCompanion_settings.RandomRecall then
							RandomCompanion.Pet();
						else
							CallCompanion("CRITTER", RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter);
						end
					else
						--Wait 2 seconds and see if you're still not busy
						debugmsg("Recalling critter in 2 seconds", 2);
						RandomCompanion.NotBusy = true;
						RandomCompanion.RecallTime = GetTime() + 2;
					end
				else
					RandomCompanion.NotBusy = nil;
					RandomCompanion.RecallTime = GetTime() + 2;
					debugmsg("Not recalling critter because you are busy.", 1);
				end
			end
		end
		
		if RandomCompanion_settings.AutoDismiss and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
			debugmsg("Checking to dismiss pets", 1);
			local creatureid, name, spellid, icon, active = GetCompanionInfo("CRITTER", RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter);
			if active then
				--Check to see if you're stealthed, prowling, or shadowmelded
				local stealth = UnitBuff("player", RC_STRING_STEALTH);
				local prowl = UnitBuff("player", RC_STRING_PROWL);
				local shadowmeld = UnitBuff("player", RC_STRING_SHADOWMELD);
				if (stealth or prowl or shadowmeld) and UnitIsPVP("player") then
					debugmsg("Auto-dismissing critter now", 2);
					local activecritter = RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter;
					RandomCompanion.origDismissCompanion("CRITTER");
				end
			end
		end
	end
end

--Hook into DismissCompanion() so that we don't auto-recall critters that were dismissed on purpose
RandomCompanion.origDismissCompanion = DismissCompanion;
DismissCompanion = function(...)
	local companiontype = ...;
	
	if (string.lower(companiontype) == "critter") then
		RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter = nil;
		RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
		debugmsg("Critter manually dismissed. Will not auto-recall.", 4);
	end
	return RandomCompanion.origDismissCompanion(...);
end

function RandomCompanion.Slash(...)
	local parsedcmd = ...;
	local cmd = "";
	local options = "";
	
	debugmsg("cmd: " .. parsedcmd, 1);
	
	if (...) then
		--Parse any macro conditionals such as [modifier:ctrl] or [combat]
		parsedcmd = SecureCmdOptionParse(...);
	end
	
	debugmsg("parsed cmd: " .. parsedcmd, 1);
	
	if string.find(parsedcmd, " ") ~= nil then
		cmd = string.sub(parsedcmd, 0, string.find(parsedcmd, " ") - 1);
		options = string.sub(parsedcmd, string.find(parsedcmd, " ") + 1);
	elseif parsedcmd ~= nil then
		cmd = parsedcmd;
	end
	
	if string.lower(cmd) == "autorecall" then
		if (RandomCompanion_settings.AutoRecall or options == "off") and string.lower(options) ~= "on" then
			RandomCompanion_settings.AutoRecall = false;
			RandomCompanion_settings.RandomRecall = false;
			DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion vanity pet recall disabled");
			if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
				RandomCompanion.OnUpdateTime = GetTime() + 2;
				RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
			else
				RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
			end
		else
			RandomCompanion_settings.AutoRecall = true;
			RandomCompanion_settings.RandomRecall = false;
			DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion vanity pet recall enabled");
			if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
				RandomCompanion.OnUpdateTime = GetTime() + 2;
				RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
			else
				RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
			end
		end
	elseif string.lower(cmd) == "autodismiss" then
		if (RandomCompanion_settings.AutoDismiss or options == "off") and string.lower(options) ~= "on" then
			RandomCompanion_settings.AutoDismiss = false;
			DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion vanity pet auto-dismiss disabled");
			if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
				RandomCompanion.OnUpdateTime = GetTime() + 2;
				RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
			else
				RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
			end
		else
			RandomCompanion_settings.AutoDismiss = true;
			DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion vanity pet auto-dismiss enabled");
			if (RandomCompanion_settings.AutoRecall or RandomCompanion_settings.RandomRecall or RandomCompanion_settings.AutoDismiss) and RandomCompanion_settings[GetRealmName()][UnitName("player")].ActiveCritter then
				RandomCompanion.OnUpdateTime = GetTime() + 2;
				RandomCompanion.EventFrame:SetScript("OnUpdate", RandomCompanion.OnUpdate);
			else
				RandomCompanion.EventFrame:SetScript("OnUpdate", nil);
			end
		end
	elseif string.lower(cmd) == "debug" then	
		if (options ~= "" and tonumber(options) == 0) or (RandomCompanion_settings.DEBUG and (tonumber(options) == nil or options == "")) then
			if (tonumber(options) ~= nil) then
				RandomCompanion_settings.DEBUGLevel = options;
			end
			RandomCompanion_settings.DEBUG = false;
			DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion debug disabled");
		else
			if (tonumber(options) ~= nil) then
				RandomCompanion_settings.DEBUGLevel = options;
			end
			if tonumber(RandomCompanion_settings.DEBUGLevel) < 1 or tonumber(RandomCompanion_settings.DEBUGLevel) == nil then
				RandomCompanion_settings.DEBUGLevel = 1;
			end
			RandomCompanion_settings.DEBUG = true;
			DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion debug enabled - Level " .. RandomCompanion_settings.DEBUGLevel);
		end
	elseif string.lower(cmd) == "mount" then
		RandomCompanion.Mount(options);
	elseif string.lower(cmd) == "pet" then
		RandomCompanion.Pet(options);
	elseif string.lower(cmd) == "petdismiss" or string.lower(cmd) == "dismiss" then
		RandomCompanion.PetDismiss(options);
	elseif string.lower(cmd) == "randomrecall" then
		if (RandomCompanion_settings.RandomRecall or options == "off") and string.lower(options) ~= "on" then
			RandomCompanion_settings.AutoRecall = false;
			RandomCompanion_settings.RandomRecall = false;
			DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion random vanity pet recall disabled");
		else
			RandomCompanion_settings.AutoRecall = false;
			RandomCompanion_settings.RandomRecall = true;
			DEFAULT_CHAT_FRAME:AddMessage("RandomCompanion random vanity pet recall enabled");
		end
	elseif string.lower(cmd) == "reload" then
		RandomCompanion.Initialize(options);
	elseif string.lower(cmd) == "status" then
		RandomCompanion.Status(options);
	else --Show help
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFF00 RandomCompanion version " .. VERSION);
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFF00 Bind a key from the keybinding menu or...");
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFF00 \"/rc mount\" to choose a random mount");
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFF00 \"/rc mount ground\" to choose a random ground mount in a flyable zone");
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFF00 \"/rc pet\" to choose a random vanity pet");
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFF00 \"/rc autorecall\" to toggle automatically recalling your vanity pet after resurrecting, changing zones, or taking flight paths");
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFF00 \"/rc randomrecall\" to toggle recalling a random vanity pet after resurrecting, changing zones, or taking flight paths");
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFF00 \"/rc dismiss\" to dismiss your current vanity pet");
		DEFAULT_CHAT_FRAME:AddMessage("|cFFFFFF00 \"/rc autodismiss\" to toggle automatically dismissing your vanity pet when you are stealthed and flagged for PVP");
	end
	
	if (RandomCompanionOptionsCB_AutoRecall) then
		RandomCompanionOptionsCB_AutoRecall:SetChecked(RandomCompanion_settings.AutoRecall);
	end
	if (RandomCompanionOptionsCB_AutoDismiss) then
		RandomCompanionOptionsCB_AutoDismiss:SetChecked(RandomCompanion_settings.AutoDismiss);
	end
	if (RandomCompanionOptionsCB_RandomRecall) then
		RandomCompanionOptionsCB_RandomRecall:SetChecked(RandomCompanion_settings.RandomRecall);
	end
end

SlashCmdList["RANDOMMOUNT"] = RandomCompanion.Mount;
SLASH_RANDOMMOUNT1 = "/randommount";
SLASH_RANDOMMOUNT2 = "/rmount";
SLASH_RANDOMMOUNT3 = "/rmt";

SlashCmdList["RANDOMPET"] = RandomCompanion.Pet;
SLASH_RANDOMPET1 = "/randompet";
SLASH_RANDOMPET2 = "/rpet";
SLASH_RANDOMPET3 = "/rpt";

SlashCmdList["PETDISMISS"] = RandomCompanion.PetDismiss;
SLASH_PETDISMISS1 = "/petdismiss";
SLASH_PETDISMISS2 = "/pdismiss";

SlashCmdList["RCSTATUS"] = RandomCompanion.Status;
SLASH_RCSTATUS1 = "/rcstatus";

SlashCmdList["RCDEBUG"] = RandomCompanion.Debug;
SLASH_RCDEBUG1 = "/rcdebug";

SlashCmdList["RCRELOAD"] = RandomCompanion.Initialize;
SLASH_RCRELOAD1 = "/rcreload";

SlashCmdList["RC"] = RandomCompanion.Slash;
SLASH_RC1 = "/rc";
SLASH_RC2 = "/randomcompanion";

RandomCompanion.EventFrame = CreateFrame("Frame", "RandomCompanionFrame");
RandomCompanion.EventFrame:SetScript("OnEvent", RandomCompanion.Event);
RandomCompanion.EventFrame:RegisterEvent("COMPANION_LEARNED");
RandomCompanion.EventFrame:RegisterEvent("COMPANION_UPDATE");
RandomCompanion.EventFrame:RegisterEvent("PLAYER_LOGIN");
RandomCompanion.EventFrame:RegisterEvent("PLAYER_CONTROL_GAINED");
RandomCompanion.EventFrame:RegisterEvent("PLAYER_UNGHOST");
RandomCompanion.EventFrame:RegisterEvent("PLAYER_ALIVE");
RandomCompanion.EventFrame:RegisterEvent("PLAYER_ENTERING_WORLD");
